function [est] = MY_est(Y,spec,init)

% This function estimates the Macro-Yields model with surveys (and its
% variants) and produces forecasts for all the variables in the system

% Source: "Does real-time macroeconomic information help to predict interest rates?" 
% by Alberto Caruso and Laura Coroneo, Journal of Money, Credit and
% Banking, Forthcoming

% Input:
% Y:        T x N matrix of data
% spec:     structure with 
%           - mat   1 x m vector of maturities
%           - r     number of total factors
% init      initial values (optional)

% Output
% est:      structure with 
%           - F_hat fitted and forecasted factors
%           - X_sm fit and forecast for all the variables in the system
%           - Ahat state matrix estimate
%           - Chat factor loadings estimate
%           - Qhat estimated state variance
%           - Rhat estimated idiosincratic variance

% Contact Author: Laura Coroneo
% e-mail address: laura.coroneo@york.ac.uk
% Release date: 24/10/22

[T, N] = size(Y);
n = size(spec.mat,2);

% quarterly series indicators
das = sum(isnan(Y(1:20,:)));
spec.q = logical(das>0); 
spec.nq = sum(spec.q);
nq = spec.nq;

%% Initialization
if nargin<3  % if no initial values are provided, fully initialise the coefficients
    
    % drop missings at the end of the sample for intialization
    das = sum(isnan(Y(:,~(spec.q|isnan(Y(1,:)))) ));
    m = max(das);
    YY = Y(1:end-m,:);

    % initalise all the coefficients
    [A, C, Q, R, initx, initV] = initial (YY,spec);

    % time varying A and Q in the out of sample part
    for t=1:m
        A(:,:,T-m+t)= A(:,:,T-m+t-6);
        Q(:,:,T-m+t)= Q(:,:,T-m+t-6);
    end
else
    % use the previous values to initialize the parameters with
    A = init.Ahat;
    C = init.Chat;
    Q = init.Qhat;
    R = init.Rhat;
    initx = init.initx;
    initV = init.initV;
end

%% Restrictions

% linear restrictions on the observation equation of yields
r = spec.r ;
RRR_y = blkdiag(eye(3), eye(r-3), 1, eye((nq>0)*r),eye(n), eye(N-n));
qqq_y = [C(1:n,1:3) zeros(n,r-3)  zeros(n,1)  zeros(n, (nq>0)*r) ...
    eye(n) zeros(n,N-n)];

% linear restrictions on the observation equation of macro
RRR_x = blkdiag(0*eye(3), zeros(r-3), 0, eye((nq>0)*r), eye(n), eye(N-n));
qqq_x = [ zeros(N-n-nq,r+1) zeros(N-n-nq,(nq>0)*r) zeros(N-n-nq,n) eye(N-n-nq) zeros(N-n-nq, nq)];

% linear restrictions on the observation equation quarterly macro
RRR_q = blkdiag(eye(r),0, zeros((nq>0)*(3)), eye(r-3+n), eye(N-n-nq), eye(nq));
qqq_q = [ zeros(nq,r) zeros(nq,(nq>0)*(r+1) )  zeros(nq,N-nq) eye(nq)];

% linear restriction on state equation: H2 * A(:) = q2
H2 = kron(eye(r+1), [zeros(1, r) 1]);
q2 = [zeros(r,1); 1];

%% EM algorithm

% the number of factors is r + intercept(1) + r (if surveys) + idiosincratic(N)
rr = r + 1 + r*(nq>0)+ N;

% initialize the parameters for the algoritm
num_iter = 0;
LL = -inf;
converged = 0;

while (num_iter < 300) & ~converged
    
    %%% E step
    [beta, gamma_t, delta_t, gamma1, gamma2, initx_new, initV_new, loglik, xsmooth]  = ...
        Estep(Y', A, C, Q, R, initx, initV);
    
    
    %%% M step
    
    % measurement equation
    S = zeros(N,N,T);
    for t = 1 : T
        S(:,:,t) = diag(double(~isnan(Y(t,:))));
    end
    C_new = restr_corr(delta_t,gamma_t,S,RRR_y,qqq_y,RRR_x,qqq_x, RRR_q,qqq_q);
   
    R_new = zeros(N,N);
    for t = 1 : T
        y_t = Y(t,:);
        y_t(:,isnan(y_t)) = 0;
        R_new = R_new + ( y_t'*y_t - delta_t(:,:,t) * C_new'*S(:,:,t) - ...
            S(:,:,t) * C_new * delta_t(:,:,t)' + S(:,:,t) * C_new * gamma_t(:,:,t) * C_new' * S(:,:,t)+...
            (eye(N)-S(:,:,t)) * R *  (eye(N)-S(:,:,t)) ) ;
    end
    R_new = diag(diag(R_new))/T;
    
    % state equation
    A_new = beta(1:r+1,1:r+1) * pinv(gamma1(1:r+1,1:r+1));
    A_new(r+1,1:r) = 0;
    A_new(r+1,r+1) = 1;
    
    Q_new = (gamma2(1:r+1,1:r+1) - beta(1:r+1,1:r+1)*A_new'- ...
        A_new*beta(1:r+1,1:r+1)'+ A_new * gamma1(1:r+1,1:r+1) *A_new') / (T-1);
   
        for i =r+1+(nq>0)*r+1 : rr
            a(i-(r+1+(nq>0)*r)) = beta(i,i) * pinv(gamma1(i,i));
            q(i-(r+1+(nq>0)*r)) = (gamma2(i,i) - beta(i,i)*a(i-(r+1+(nq>0)*r))- ...
       a(i-(r+1+(nq>0)*r))*beta(i,i)'+ a(i-(r+1+(nq>0)*r)) * gamma1(i,i) *a(i-(r+1+(nq>0)*r))) / (T-1);
        end
        A_new = blkdiag(A_new, diag(a));
        Q_new = blkdiag(Q_new, diag(q));
    
    
    if nq>0
        w = repmat([ 1 1/2 1/3]',ceil(T/3),1);
        w = w(1:T);
        
        iota = repmat([0 1/2 2/3]',ceil(T/3),1);
        iota = iota(1:T);
        
        AA_new = NaN(size(A_new,1)+r,size(A_new,1)+r,T);
        QQ_new = NaN(size(Q_new,1)+r,size(Q_new,1)+r,T);
        for t=1:T
            itemp =eye(r+1+r);
            itemp (r+2:end,1:r) = blkdiag(w(t) * eye(r));
            
            AA_new(:,:,t) =  blkdiag(A_new(1:r+1,1:r+1), iota(t)*eye(r), A_new(r+2:end,r+2:end));
            AA_new(1:r+r+1,1:r+r+1,t)= itemp * AA_new(1:r+r+1,1:r+r+1,t);
            
            QQ_new(:,:,t) = blkdiag(Q_new(1:r+1,1:r+1), 0 * eye(r), Q_new(r+2:end,r+2:end));
            QQ_new(1:r+r+1,1:r+r+1,t)= itemp * QQ_new(1:r+r+1,1:r+r+1,t)*itemp';
            
        end
    else
        AA_new =repmat(A_new,1,1,T);
        
        QQ = ( gamma2 - AA_new(:,:,2)*beta') / (T-1);
        Q_new = blkdiag(Q_new(1:r,1:r), 1e-20, diag(diag(QQ(end-N+1:end,end-N+1:end))));    
        QQ_new =repmat(Q_new,1,1,T);
    end
    
     
    %%% check convergence
    if num_iter>0
        [converged, decrease] = em_converged(LL(end), LL(end-1), 1e-3,1);
    end
    
    
        num_iter =  num_iter + 1;
        LL = [LL loglik];
        previous_loglik = loglik;
        A = AA_new; C =C_new;  Q =QQ_new; R=R_new; initx=initx_new; initV =initV_new;
  
end

%% add NaNs for the forecast

x_NAN = [Y; nan(24, N)];

for t= T-12 : size(x_NAN,1)
    A(:,:,t)= A(:,:,t-3);
    Q(:,:,t)= Q(:,:,t-3);
end

[xsmooth, Vsmooth, VVsmooth, loglik] = runKF_tv(x_NAN',A, C, Q, R, initx, initV);

%% Collect results
est.F_hat= xsmooth';
est.X_sm = xsmooth(:,:)'*C(:,:)';
est.Ahat = A;
est.Chat = C;
est.Qhat = Q;
est.Rhat = R;
est.initx = initx;
est.initV = initV;
est.Vsmooth = Vsmooth;
est.VVsmooth = VVsmooth;
est.loglik = loglik;
