% "Does real-time macroeconomic information help to predict interest rates?" 
% by Alberto Caruso and Laura Coroneo, Journal of Money, Credit and
% Banking, Forthcoming

% Code to estimate the 4 models presented in the paper:
% OY        % only-yields model
% OY_SPF    % only-yields model with surveys
% MY        % macro-yields model in real-time
% MY_SPF    % macro-yields model in real-time with surveys

% This sample code reproduces Figure 5 in the paper

% Note: the analysis in the paper includes one additional macroeconomic
% variable (PMI Composite Index (NAPM), which can be dowloaded from Refinitiv 
% using the code USCNFBUSQ)

% Contact Author: Laura Coroneo
% e-mail address: laura.coroneo@york.ac.uk
% Release date: 24/10/22

clear; clc;
close all;

%% Date for which we want the forecasts
target_date = 20130228;

% maturities
spec.mat = [3	6	12	24	36	48	60	84	120];
m = length(spec.mat);

%% Data before forward guidance announcement (Jan 2012)
data = readtable('data_jan_2012.txt');
data_ML = data{:,2:end};
data_ML_pre = [data_ML(:,1:m) standardise(data_ML(:,m+1:end))];

dd = data.date;
N = size(data_ML,2);

%% Data after forward guidance announcement (Feb 2012)
data = readtable('data_feb_2012.txt');
data_ML = data{:,2:end};
data_ML_post = [data_ML(:,1:m) standardise(data_ML(:,m+1:end))];

%% Estimate the models
disp('Estimating the OY model...');
spec.r = 3;
Res_OY_pre = MY_est(data_ML_pre(:,1:m),spec);
Res_OY_post = MY_est(data_ML_post(:,1:m),spec,Res_OY_pre);

disp('Estimating the OY+SPF model...');
Res_OY_SPF_pre = MY_est(data_ML_pre(:,[1:m N-1:N]),spec);
Res_OY_SPF_post = MY_est(data_ML_post(:,[1:m N-1:N]),spec, Res_OY_SPF_pre);

disp('Estimating the MY model...');
spec.r = 4;
Res_MY_pre = MY_est(data_ML_pre(:,1:end-2),spec);
Res_MY_post = MY_est(data_ML_post(:,1:end-2),spec, Res_MY_pre);

disp('Estimating the MY+SPF model...');
Res_MY_SPF_pre = MY_est(data_ML_pre,spec);
Res_MY_SPF_post = MY_est(data_ML_post,spec, Res_MY_SPF_pre);

%% Collect forecasts 

% find the indicator for the target date
ind = find(datenum(dd)>=datenum(target_date),1,'First');

% forecasts made in Jan 2012
FF_pre = [ Res_OY_pre.X_sm(ind,1:m); Res_OY_SPF_pre.X_sm(ind,1:m); ...
    Res_MY_pre.X_sm(ind,1:m);    Res_MY_SPF_pre.X_sm(ind,1:m)];

% forecasts made in Feb 2012
FF_post = [ Res_OY_post.X_sm(ind,1:m); Res_OY_SPF_post.X_sm(ind,1:m); ...
    Res_MY_post.X_sm(ind,1:m);    Res_MY_SPF_post.X_sm(ind,1:m)];

%% Load realised values for interest rates
data = readtable('data_all.txt');
yield_all = data{:,2:end};

%% Figure

figure
subplot(2,1,1)
h1=plot(1:m,[yield_all(ind,:); max(FF_pre,0)], 'LineWidth', 3);
set(h1,{'LineStyle'},{'-';'-';'--';'-';'--'});
set(h1,{'Color'},{'k';'r';'r';'b';'b'});
set(gca,'Xtick',1:length(spec.mat))
set(h1,{'Marker'},{'none';'square';'square'; '+';'+'})
ax = gca;
ax.FontSize = 14;
set(gca,'XTickLabel',{'3m', '6m', '1y', '2y','3y','4y','5y', '7y', '10y'});
xlabel('Maturity');
title('Forecast 13-step ahead before a FG announcement (Jan 2012)');
legend({'Actual','OY', 'OY+S', 'MY RT', 'MY RT+S'}, 'Orientation','vertical','Location','northwest');


subplot(2,1,2)
h1=plot(1:m,[yield_all(ind,:); max(FF_post,0)], 'LineWidth', 3);
set(h1,{'LineStyle'},{'-';'-';'--';'-';'--'});
set(h1,{'Color'},{'k';'r';'r';'b';'b'});
set(gca,'Xtick',1:length(spec.mat))
set(h1,{'Marker'},{'none';'square';'square'; '+';'+'})
ax = gca;
ax.FontSize = 14;
set(gca,'XTickLabel',{'3m', '6m', '1y', '2y','3y','4y','5y', '7y', '10y'});
xlabel('Maturity');
title('Forecast 12-step ahead, after FG announcement (Feb 2012)');
legend({'Actual','OY', 'OY+S', 'MY RT', 'MY RT+S'}, 'Orientation','vertical','Location','northwest');
