function[A, C, Q, R, initx, initV] = initial(Y,spec)

% This function initialises all the state-space system parameters

% Source: "Does real-time macroeconomic information help to predict interest rates?" 
% by Alberto Caruso and Laura Coroneo, Journal of Money, Credit and
% Banking, Forthcoming


% Input
% Y:        TxN matrix of data
% spec:     structure with 
%           - mat   1 x m vector of maturities
%           - q     1 x N matrix with indicators for quarterly
%           - r     number of total factors
%           - nq    number of quarterly series

% Output        
% A         initial state matrix estimate
% C         initial factor loadings estimate
% Q         initial state variance
% R         initial idiosincratic variance
% initx     initial state values
% initV     initial state variance

% Contact Author: Laura Coroneo
% e-mail address: laura.coroneo@york.ac.uk
% Release date: 24/10/22

[T, N] = size(Y);
n = size(spec.mat,2);
r = spec.r;

%% Initialise factors

% initial yield curve factors
[yield_factors, coef]= NS_fact(spec.mat, Y(:,1:n), 0.0609);
resid   = Y(:,1:n) - yield_factors * coef';

% initial quarterly yield factors
if spec.nq>0
    NQ = ceil(T/3);
    for i=1:3
        tempq = reshape([yield_factors(:,i); NaN(3-rem(T,3)-3*(rem(T,3)==0),1) ], 3, NQ);
        temp1 =  [tempq(1,:); nanmean(tempq(1:2,:)); nanmean(tempq)];
        yield_factors_q(:,i) = temp1(:);
    end
    yield_factors_q = yield_factors_q(1:T,:);
end

% macro factors
if r>3
    % coefficients of the macro variables on the NS factors and intercept
    FF = [yield_factors ones(size(yield_factors,1),1)];
    Y_temp = Y(:,~spec.q);
    macro_coef = Y_temp(:,n+1:end)' * FF * inv(FF' * FF);

    % residuals of the macro variables
    resid = Y_temp(:,n+1:end)- FF * macro_coef';

    % principal components from the residuals
    [v,d]     = eigs(cov(resid), r-3, 'lm');
    G         = -resid * v * d^(-.5);

    % quarterly macro factors
    if spec.nq>0
        for i = 1: size(G,2)
            tempq = reshape([G(:,i); NaN(3-rem(T,3)-3*(rem(T,3)==0),1) ], 3, NQ);
            temp1 =  [tempq(1,:); mean(tempq(1:2,:)); mean(tempq)];
            GG_q(:,i) = temp1(:);
        end
        yield_factors_q = [yield_factors_q GG_q(1:T,:)];
    end

    % matrix of yield factors
    F = [yield_factors G];

else
    F = yield_factors;
end

% include the intercept
F = [F ones(size(yield_factors,1),1)];

%% Observation equation

% recompute the loadings
C = (inv(F'*F)*F'*Y(:,~spec.q))';
C(1:n,1:3)  = coef;
C(1:n,r+1)= 0;
C(1:n,3+1:end-1)= 0;

% compute the residuals
Residual = Y(:,~spec.q)-F*C';
R = diag(diag(cov(Residual)));

%% add quarterly in measurement equation
if spec.nq>0

    Y_temp = Y(:,spec.q);
    for qq=1:spec.nq
        idx_first = find(~isnan(Y_temp(:,qq)), 1, 'first' )-2;
        idx_last  = find(~isnan(Y_temp(:,qq)), 1, 'last' );

        % interpolate quarterly data (only from the first to the last NaN) and add it to the data
        temp=NaN(size(Y_temp,1),1);
        temp(idx_first:idx_last,1)=inpaint_nans(Y_temp(idx_first:idx_last,qq));

        % compute coefficients of surveys on quarterly yield factors
        if ~isempty(idx_first)
            FFq = [yield_factors_q(idx_first:idx_last,1:3) ones(idx_last-idx_first+1,1)];
            surv_coef(qq,:) = temp(idx_first:idx_last)' * FFq* inv(FFq' * FFq);
        end
    end

    % add quarterly factors
    F = [F yield_factors_q];
    C = [C zeros(size(C,1),r); zeros(spec.nq,r) surv_coef(:,[end 1:end-1]) zeros(spec.nq,r-3)];
    fit = F*C';
    residualq = Y(3:3:end,spec.q)- fit(3:3:end,spec.q);
    ind = isnan(residualq(:,end));
    R= blkdiag(R,diag(diag(cov(residualq(~ind,:)))));

    for q = 1:spec.nq
        res = repmat(residualq(:,q)', 3,1 );
        res = res(:);
        res(isnan(res))=0;
        Residual = [Residual [res; zeros(rem(T,3)-3*(rem(T,3)==0),1)]];
    end
end

%% state equation
Z = F(1:end-1,1:r+1);
z = F(2:end,1:r+1);

% OLS estimator of the VAR transition matrix
A = (inv(Z'*Z)*Z'*z)'; 

% impose the restrictions on A
A(end,1:r) = 0;
A(end,end) = 1;
e = z  - Z*A'; % VAR residuals
Q = cov(e); % VAR covariance matrix

% impose the restrictions on Q
Q(end,end)= 1e-20;
Q(1:r,end)= 0;
Q(end,1:r)= 0;

% initialize
initx = [F(1,1:3)'; zeros(r-3,1); 1];
initV = reshape(pinv(eye((r+1)*(r+1))-kron(A,A))*Q(:),r+1,r+1);
initV(r+1,1:r)=0;
initV(1:r,r+1)=0;
initV(r+1,r+1)=1e-12;

%% add the idio in the states

F = [F Residual];

% observation equation
C = [C eye(N)];
R(1:N,1:N)= exp(-12);
R = diag(diag(R));

% state equation
rho = zeros(N);
S = zeros(N);
for i = 1: N
    temp= Residual(:,i);
    rho(i,i) = (pinv(temp(1:end-1)'*temp(1:end-1))...
        *temp(1:end-1)'*temp(2:end))';
    S(i,i) = cov( temp(2:end)  - temp(1:end-1)*rho(i,i)');
end

A = blkdiag(A,rho);
Q = blkdiag(Q,S);

% initialize the states
initx = [initx; zeros(N,1)];
initS = diag(1./diag(eye(size(rho,1))-rho.^2)) .*S;
initV = blkdiag(initV, initS);

%% add quarterly in state equation
if spec.nq>0
    w = repmat([ 1 1/2 1/3]',NQ,1);
    w = w(1:T);

    iota = repmat([0 1/2 2/3]',ceil(T/3),1);
    iota = iota(1:T);

    AA = NaN(size(A,1)+r,size(A,1)+r, T);
    QQ = NaN(size(Q,1)+r,size(Q,1)+r, T);
    for t=1:T
        itemp =eye(r+1+r);
        itemp (r+2:end,1:r) = w(t) * eye(r);

        AA(:,:,t) =  blkdiag(A(1:r+1,1:r+1), iota(t)*eye(r), A(r+2:end,r+2:end));
        AA(1:r+r+1,1:r+r+1,t)= itemp * AA(1:r+r+1,1:r+r+1,t);

        QQ(:,:,t) = blkdiag(Q(1:r+1,1:r+1), 0 * eye(r), Q(r+2:end,r+2:end));
        QQ(1:r+r+1,1:r+r+1,t)= itemp * QQ(1:r+r+1,1:r+r+1,t)*itemp';
    end
    initx = [initx(1:r+1); initx(1:r); initx(r+2:end)];
    initV = blkdiag(initV(1:r+1,1:r+1), 0* eye(r),initV(r+2:end,r+2:end));
    clear A;

    A = AA;
    Q = QQ;
else
    A = repmat(A,1,1,T);
    Q = repmat(Q,1,1,T);
end